@@ -1,6 +1,5 @@
 # XXX todo
 # regions
-# paths
 package Prima::PS::PDF;
 
 use strict;
@@ -78,6 +77,7 @@ sub change_transform
 	my @cr = $self-> clipRect;
 	my @sc = $self-> scale;
 	my $ro = $self-> rotate;
+	my $rg = $self-> region;
 
 	$cr[2] -= $cr[0];
 	$cr[3] -= $cr[1];
@@ -85,7 +85,7 @@ sub change_transform
 	my $doTR   = grep { $_ != 0 } @tp;
 	my $doSC   = grep { $_ != 0 } @sc;
 
-	if ( !$doClip && !$doTR && !$doSC && !$ro) {
+	if ( !$doClip && !$doTR && !$doSC && !$ro && !$rg) {
 		$self-> emit_content('q') if $gsave;
 		return;
 	}
@@ -96,8 +96,9 @@ sub change_transform
 
 	$self-> emit_content('Q') unless $gsave;
 	$self-> emit_content('q');
-	$self-> emit_content("@cr W") if $doClip;
+	$self-> emit_content("@cr W n") if $doClip;
 	$self-> emit_content("1 0 0 1 @tp cm") if $doTR;
+	$self-> emit_content($rg-> apply_offset . " n") if $rg && !$doClip;
 	$self-> emit_content("$sc[0] 0 0 $sc[1] 0 0 cm") if $doSC;
 	if ($ro != 0) {
 		my $sin1 = sin($ro);
@@ -734,62 +735,6 @@ sub compress
 	$self-> {compress} = $_[1];
 }
 
-our $PI = 3.14159265358979323846264338327950288419716939937510;
-our $RAD = 180.0 / $PI;
-
-# L.Maisonobe 2003
-# http://www.spaceroots.org/documents/ellipse/elliptical-arc.pdf
-sub arc2cubics
-{
-	my ( $self, $x, $y, $dx, $dy, $start, $end) = @_;
-
-	my ($reverse, @out);
-	($start, $end, $reverse) = ( $end, $start, 1 ) if $start > $end;
-
-	push @out, $start;
-	# see defects appearing after 45 degrees:
-	# https://pomax.github.io/bezierinfo/#circles_cubic
-	while (1) {
-		if ( $end - $start > 45 ) {
-			push @out, $start += 45;
-			$start += 45;
-		} else {
-			push @out, $end;
-			last;
-		}
-	}
-	@out = map { $_ / $RAD } @out;
-
-	my $rx = $dx / 2;
-	my $ry = $dy / 2;
-
-	my @cubics;
-	for ( my $i = 0; $i < $#out; $i++) {
-		my ( $a1, $a2 ) = @out[$i,$i+1];
-		my $b           = $a2 - $a1;
-		my ( $sin1, $cos1, $sin2, $cos2) = ( sin($a1), cos($a1), sin($a2), cos($a2) );
-		my @d1  = ( -$rx * $sin1, -$ry * $cos1 );
-		my @d2  = ( -$rx * $sin2, -$ry * $cos2 );
-		my $tan = sin( $b / 2 ) / cos( $b / 2 );
-		my $a   = sin( $b ) * (sqrt( 4 + 3 * $tan * $tan) - 1) / 3;
-		my @p1  = ( $rx * $cos1, $ry * $sin1 );
-		my @p2  = ( $rx * $cos2, $ry * $sin2 );
-		my @points = (
-			@p1,
-			$p1[0] + $a * $d1[0],
-			$p1[1] - $a * $d1[1],
-			$p2[0] - $a * $d2[0],
-			$p2[1] + $a * $d2[1],
-			@p2
-		);
-		$points[$_] += $x for 0,2,4,6;
-		$points[$_] += $y for 1,3,5,7;
-		@points[0,1,2,3,4,5,6,7] = @points[6,7,4,5,2,3,0,1] if $reverse;
-		push @cubics, \@points;
-	}
-	return \@cubics;
-}
-
 sub arc
 {
 	my ( $self, $x, $y, $dx, $dy, $start, $end) = @_;
@@ -1065,7 +1010,7 @@ sub bars
 	for ( $i = 0; $i < $c; $i += 4) {
 		$z .= "h @a[$i,$i+1] " . ($a[$i+2] - $a[$i]) . ' ' . ($a[$i+3] - $a[$i+1]) . " re f\n";
 	}
-	$self-> stroke( $z);
+	$self-> fill( $z);
 }
 
 sub clear
@@ -1316,30 +1261,34 @@ sub new_path
 	return Prima::PS::PDF::Path->new(@_);
 }
 
-package Prima::PS::PDF::Path;
-use base qw(Prima::Drawable::Path);
-
-sub entries
+sub region
 {
-	my $self = shift;
-	unless ( $self->{entries} ) {
-		local $self->{stack} = [];
-		local $self->{curr}  = { matrix => [ $self-> identity ] };
-		my $c = $self->{commands};
-		$self-> {entries} = [];
-		for ( my $i = 0; $i < @$c; ) {
-			my ($cmd,$len) = @$c[$i,$i+1];
-			$self-> can("_$cmd")-> ( $self, @$c[$i+2..$i+$len+1] );
-			$i += $len + 2;
-		}
-		$self->{last_matrix} = $self->{curr}->{matrix};
+	return $_[0]->{region} unless $#_;
+	my ( $self, $region ) = @_;
+	if ( $region && !UNIVERSAL::isa($region, "Prima::PS::PDF::Region")) {
+		warn "Region is not a Prima::PS::PDF::Region";
+		return undef;
 	}
-	return $self-> {entries};
+	$self->{clipRect} = [0,0,0,0];
+	$self->{region} = $region;
+	$self-> change_transform;
 }
 
-sub emit { push @{shift->{entries}}, join(' ', @_) }
+package
+	Prima::PS::PDF::Path;
+use base qw(Prima::PS::Drawable::Path);
 
-sub last_point { @{$_[0]->{last_point} // [0,0]} }
+my %dict = (
+	lineto    => 'l',
+	moveto    => 'm',
+	curveto   => 'c',
+	stroke    => 'S',
+	closepath => 'h',
+	fill_alt  => 'f*',
+	fill_wind => 'f',
+);
+
+sub dict { \%dict }
 
 sub set_current_point
 {
@@ -1348,105 +1297,37 @@ sub set_current_point
 	$self-> {move_is_line} = 1;
 }
 
-sub _open
+sub region
+{
+	my ($self, $mode) = @_;
+	my $path = join "\n", @{$self-> entries};
+	$path .= ' h' unless $path =~ /h$/;
+	$path .= ' W';
+	$path .= '*' if ($mode // fm::Winding) & fm::Alternate;
+	return Prima::PS::PDF::Region->new( $path );
+}
+
+package
+	Prima::PS::PDF::Region;
+use base qw(Prima::PS::Drawable::Region);
+
+sub other { UNIVERSAL::isa($_[0], "Prima::PS::PDF::Region") ? $_[0] : () }
+
+sub equals
 {
 	my $self = shift;
-	$self-> {move_is_line} = 0;
-	$self->emit('')
+	my $other = other(shift) or return;
+	return $self->{path} eq $other->{path};
 }
 
-sub _close     { shift->emit('h') }
-
-sub  _moveto
-{
-	my ( $self, $mx, $my, $rel) = @_;
-	($mx, $my) = $self-> canvas-> pixel2point( $mx, $my );
-	($mx, $my) = $self->matrix_apply($mx, $my);
-	my ($lx, $ly) = $rel ? $self->last_point : (0,0);
-	$lx += $mx;
-	$ly += $my;
-	@{$self-> {last_point}} = ($lx, $ly);
-	$self-> emit($lx, $ly, 'm');
-}
-
-sub _line
-{
-	my ( $self, $line ) = @_;
-	my @line = $self-> canvas-> pixel2point( @$line );
-	@line = @{ $self-> matrix_apply( \@line ) };
-	$self-> set_current_point( shift @line, shift @line );
-	@{$self-> {last_point}} = @line[-2,-1];
-	for ( my $i = 0; $i < @line; $i += 2 ) {
-		$self->emit(@line[$i,$i+1], 'l');
-	}
-}
-
-sub conic2curve
-{
-	my ($x0, $y0, $x1, $y1, $x2, $y2) = @_;
-	my (@cp1, @cp2);
-	$cp1[0] = $x0 + 2 / 3 * ($x1 - $x0);
-	$cp1[1] = $y0 + 2 / 3 * ($y1 - $y0);
-	$cp2[0] = $x2 + 2 / 3 * ($x1 - $x2);
-	$cp2[1] = $y2 + 2 / 3 * ($y1 - $y2);
-	return @cp1, @cp2, $x2, $y2;
-}
-
-sub _spline
-{
-	my ( $self, $points, $options ) = @_;
-	my @p = $self-> canvas-> pixel2point( @$points );
-	@p = @{ $self-> matrix_apply( \@p ) };
-
-	$options->{degree} //= 2;
-	return if $options->{degree} > 3;
-	my @p0 = @p[0,1];
-	$self-> set_current_point( @p0 );
-	if ( $options->{degree} == 2 ) {
-		for ( my $i = 2; $i < @p; $i += 4 ) {
-			my @pp = conic2curve( @p0, @p[$i .. $i + 3] );
-			$self->emit(@pp, 'c');
-			@p0 = @pp[-2,-1];
-		}
-	} else {
-		for ( my $i = 2; $i < @p; $i += 4 ) {
-			my @pp = @p[$i .. $i + 5];
-			$self->emit(@pp, 'c');
-		}
-	}
-}
-
-sub _arc
-{
-	my ( $self, $from, $to, $rel ) = @_;
-	my $cubics = $self->canvas->arc2cubics( 0, 0, 1, 1, $from, $to);
-
-	if ( $rel ) {
-		my ($lx,$ly) = $self->last_point;
-		my $pts = $cubics->[0];
-		my $m = $self->{curr}->{matrix};
-		my @s = $self->matrix_apply( $pts->[0], $pts->[1]);
-		$m->[4] += $lx - $s[0];
-		$m->[5] += $ly - $s[1];
-	}
-	my @p = map { $self-> matrix_apply( $_ ) } @$cubics;
-	$self-> set_current_point( @{$p[0]}[0,1] );
-	$self-> emit( @{$_}[2..7], 'c') for @p;
-}
-
-sub stroke
+sub combine
 {
 	my $self = shift;
-	$self-> canvas-> stroke( join("\n", @{ $self->entries }, 'S' ));
+	my $other = other(shift) or return;
+	$self->{path} .= "\n" . $other->apply_offset;
 }
 
-sub fill
-{
-	my ( $self, $fillMode ) = @_;
-	$fillMode //= $self->canvas->fillMode;
-	$fillMode = ((($fillMode & fm::Winding) == fm::Alternate) ? 'f*' : 'f');
-	$self-> canvas-> fill( join("\n", @{ $self->entries }, $fillMode ));
-}
+sub is_empty { shift->{path} !~ /[Sf]/ }
 
 1;
 
