@@ -30,21 +30,15 @@ sub open(*;$*)
 {
 	my ( $handle, @p ) = @_;
 	goto NATIVE unless @p;
-	if (1 == @p) {
-		if ( $p[0] =~ m/^([\<\>\|\-\+\=\&]+)(.*)/) {
-			@p = ($1, $2);
-		} else {
-			@p = ('<', $p[0]);
-		}
-	}
+	$p[0] =~ m/^([\<\>\|\-\+\=\&]*])(.*)/ if 1 == @p;
 	my ( $mode, $what, @rest) = @p;
 	goto NATIVE if !defined($what) || ref($what);
-	goto NATIVE if $mode =~ /^[<>+]&/ || $mode =~ /^[-|]/ || $what =~ /[-|]$/;
+	goto NATIVE if $what =~ /[\-\|\=\&]/;
 
 	my $flags;
 	my @layers;
 
-	if ( $mode =~ /^([^:\s]+)([:\s].*)$/ ) {
+	if ( $mode =~ /^([^:\s]+)(.+)$/ ) {
 		$mode = $1;
 		my $binmode = $2;
 		$binmode =~ s/^\s+//;
@@ -86,7 +80,17 @@ sub open(*;$*)
 	return $ok;
 
 NATIVE:
-	goto &CORE::open;
+	no strict 'refs';
+	if ( 0 == @p ) {
+		return CORE::open($handle);
+	} elsif ( 1 == @p ) {
+		return CORE::open($handle, $p[0]);
+	} elsif ( 2 == @p ) {
+		return CORE::open($handle, $p[0], $p[1]);
+	} else {
+		my ( $x, $y ) = (shift @p, shift @p);
+		return CORE::open($handle, $x, $y, @p);
+	}
 }
 
 sub opendir(*$)
@@ -96,7 +100,7 @@ sub opendir(*$)
 		return;
 	}
 	$_[0] = open_dir( $_[1] );
-	return defined $_[0];
+	return 1;
 }
 
 sub readdir($)
@@ -119,56 +123,57 @@ sub glob
 	my $pat = shift;
 	my @pats;
 	while ( 1 ) {
-		$pat =~ m/\G"((?:[^"]|\\")*)(?<!\\)"/gcs and push @pats, $1 and next;
-		$pat =~ m/\G'((?:[^']|\\')*)(?<!\\)'/gcs and push @pats, $1 and next;
-		$pat =~ m/\G((?:\S|\\\s)+)/gcs and push @pats, $1 and next;
+		$pat =~ m/\G"([^"]*)"/gcs and push @pats, $1 and next;
+		$pat =~ m/\G'([^']*)'/gcs and push @pats, $1 and next;
+		$pat =~ m/\G(\S+)/gcs and push @pats, $1 and next;
 		$pat =~ m/\G\s+/gcs and next;
 		$pat =~ m/\G$/gcs and last;
 	}
 	my @matches = @pats;
 	@pats = ();
-	my $win32 = $^O =~ /win32/i;
-	MATCH: while ( my $q = shift @matches ) {
+	while ( my $q = shift @matches ) {
 		if ( $q =~ m/^(.*)\{([^}]*)\}(.*)$/ ) {
 			my ( $pre, $subpat, $post ) = ( $1, $2, $3 );
 			push @matches, map { "$pre$_$post" } split /,/, $subpat;
 		} elsif ( $q =~ m/^(.*)\[([^\]]*)\](.*)$/ ) {
 			my ( $pre, $subpat, $post ) = ( $1, $2, $3 );
 			push @matches, map { "$pre$_$post" } split //, $subpat;
-		} elsif ( $q =~ m/^~(\w*)(.*)/ ) {
-			my @pwent;
-			unless ( length $1 ) {
-				push @matches, ($ENV{HOME} // ($win32 ? $ENV{USERPROFILE} : undef) // '/' ) . $2;
-			} elsif (!$win32 && (@pwent = getpwnam($1)) && defined($pwent[7])) {
-				push @matches, $pwent[7] .  $2;
-			}
-		} elsif ( $q =~ m/(?<!\\)\*|\?/ ) {
-			my @paths = ('');
-			my $expanded;
-			for my $subpath ( split m{(/)}, $q ) {
-				if ( !$expanded && $subpath =~ m/(?<!\\)\*|\?/ ) {
-					$subpath =~ s/(?<!\\)\*/.*/g;
-					$subpath =~ s/(?<!\\)\?/./g;
-					$subpath = qr/$subpath/;
-					next MATCH unless Prima::sys::FS::opendir( my $dh, length($paths[0]) ? $paths[0] : '.' );
-					my $opath = pop @paths;
-					for my $e ( Prima::sys::FS::readdir $dh ) {
-						next unless $e =~ /^$subpath$/;
-						push @paths, $opath . $e;
-					}
-					Prima::Utils::closedir $dh;
-					$expanded++;
-				} else {
-					$_ .= $subpath for @paths;
-				}
-			}
-			push @matches, @paths;
-		} elsif (_e($q)) {
+		} else {
 			push @pats, $q;
 		}
 	}
+	my $win32 = $^O =~ /win32/i;
+	for $pat ( @pats ) {
+		next unless $pat =~ m/^~(\w*)(.*)/;
+		my @pwent;
+		unless ( length $1 ) {
+			$pat = ($ENV{HOME} // ($win32 ? $ENV{USERPROFILE} : undef) // '/' ) . $2;
+		} elsif ($win32 && (@pwent = getpwnam($1)) && defined($pwent[7])) {
+			$pat = $pwent[7] .  $2;
+		}
+	}
+	my (@ret, @pwent);
+	for my $q ( @pats ) {
+		my ( $dir, $mask ) = ( $q =~ m/^(.*\/)([^\/]*)/ ) ? ($1, $2) : ('.', $q);
+		if ( $mask eq '' ) {
+			next unless _e($dir);
+			push @ret, $dir;
+			next;
+		}
+		next unless Prima::sys::FS::opendir( my $dh, $dir );
+		my $cspl = sub { my $x = shift; $x =~ s/,/|/g; $x };
+		$mask =~ s/\./\\./g;
+		$mask =~ s/\*/.*/g;
+		$mask =~ s/\{([^}]*)\}/'(?:'.$cspl->($1).')'/ge;
+		$mask = qr/$mask/;
+		for my $e ( Prima::sys::FS::readdir $dh ) {
+			next unless $e =~ /^$mask$/;
+			push @ret, $e;
+		}
+		Prima::Utils::closedir $dh;
+	}
 
-	return @pats;
+	return @ret;
 }
 
 sub lstat { Prima::Utils::stat($_[0], 1) }
@@ -258,7 +263,7 @@ sub abs_path
 	return undef unless Prima::Utils::chdir($path);
 	my $realpath = Prima::Utils::getcwd();
 	if (! ((_d $cwd) && (Prima::Utils::chdir($cwd)))) {
-		croak("Cannot chdir back to $cwd: $!");
+		_croak("Cannot chdir back to $cwd: $!");
 	}
 
 	return $realpath;
@@ -278,10 +283,10 @@ Since perl win32 unicode support for files is unexistent, Prima has its own
 parallel set of functions mimicking native functions, ie open, chdir etc. This
 means that files with names that cannot be converted to ANSI (ie
 user-preferred) codepage are not visible in perl, but the functions below
-mitigate this problem.
+mitigate that problem.
 
-The module exports the unicode-aware functions from C<Prima::Utils> to override
-the core functions. Read more in L<Prima::Utils/"Unicode-aware file system functions">.
+This module exports the unicode-aware functions from C<Prima::Utils> to override
+the core functions. Read more in L<Prima::Utils/"Unicode-aware filesystem functions">.
 
 =head2 SYNOPSIS
 
