@@ -15,13 +15,14 @@ pastes its image into the clipboard.
 use strict;
 use warnings;
 
-use Prima qw(Buttons StdBitmap), Application => { name => 'Buttons sample' };
+use Prima qw(Buttons StdBitmap Application);
 
 my $w = Prima::MainWindow-> create(
-text=> "Handmade buttons",
-size => [ 300, 200],
-centered => 1,
-popupItems => [['Hallo!' => '']],
+	text=> "Handmade buttons",
+	size => [ 300, 200],
+	centered => 1,
+	popupItems => [['Hallo!' => '']],
+	designScale => [6,16],
 );
 
 $w-> insert( CheckBox =>
