@@ -48,6 +48,11 @@ extern Bool prima_find_accel( Handle self, Handle item, int * key);
 extern Bool prima_single_color_notify ( Handle self, Handle child, void * color);
 extern Bool prima_kill_all_objects( Handle self, Handle child, void * dummy);
 
+extern void  prima_init_font_mapper(void);
+extern void  prima_cleanup_font_mapper(void);
+extern PFont prima_font_mapper_save_font(const char * name);
+extern PFont prima_font_mapper_get_font(unsigned int fid );
+
 #ifdef __cplusplus
 }
 #endif
