@@ -59,55 +59,34 @@ apc_gp_done( Handle self)
 	return true;
 }
 
-void
-adjust_line_end( int  x1, int  y1, int * x2, int * y2, Bool forth)
-{
-	if ( forth) {
-		if ( x1 == *x2)
-			( y1 < *y2) ? ( *y2)++ : ( *y2)--;
-		else if ( y1 == *y2)
-			( x1 < *x2) ? ( *x2)++ : ( *x2)--;
-		else
-		{
-			//     Zinc Application Framework - W_GDIDSP.CPP COPYRIGHT (C) 1990-1998
-			long tan = ( *y2 - y1) * 1000L / ( *x2 - x1);
-			if ( tan < 1000 && tan > -1000) {
-				int dx = *x2 - x1;
-				( dx > 0) ? dx++ : dx--;
-				*x2 = x1 + dx;
-				*y2 = (int)( y1 + dx * tan / 1000L);
-			} else {
-				int dy = *y2 - y1;
-				( dy > 0) ? dy++ : dy--;
-				*x2 = (int)( x1 + dy * 1000L / tan);
-				*y2 = y1 + dy;
-			}
-			// eo Zinc
-		}
-	} else {
-		if ( x1 == *x2)
-			( y1 < *y2) ? ( *y2)-- : ( *y2)++;
-		else if ( y1 == *y2)
-			( x1 < *x2) ? ( *x2)-- : ( *x2)++;
-		else
-		{
-			//     Zinc Application Framework - W_GDIDSP.CPP COPYRIGHT (C) 1990-1998
-			long tan = ( *y2 - y1) * 1000L / ( *x2 - x1);
-			if ( tan < 1000 && tan > -1000) {
-				int dx = *x2 - x1;
-				( dx > 0) ? dx-- : dx++;
-				*x2 = x1 + dx;
-				*y2 = (int)( y1 + dx * tan / 1000L);
-			} else {
-				int dy = *y2 - y1;
-				( dy > 0) ? dy-- : dy++;
-				*x2 = (int)( x1 + dy * 1000L / tan);
-				*y2 = y1 + dy;
-			}
-			// eo Zinc
-		}
-	}
-}
+#define ADJUST_LINE_END_DEF(typ)                 \
+void                                             \
+adjust_line_end_##typ                            \
+( typ x1, typ y1, typ *x2, typ *y2)              \
+{                                                \
+	if ( x1 == *x2)                                \
+		( y1 < *y2) ? ( *y2)++ : ( *y2)--;           \
+	else if ( y1 == *y2)                           \
+		( x1 < *x2) ? ( *x2)++ : ( *x2)--;           \
+	else                                           \
+	{                                              \
+		long tan = ( *y2 - y1) * 1000L / ( *x2 - x1);\
+		if ( tan < 1000 && tan > -1000) {            \
+			typ dx = *x2 - x1;                         \
+			( dx > 0) ? dx++ : dx--;                   \
+			*x2 = x1 + dx;                             \
+			*y2 = (typ)( y1 + dx * tan / 1000L);       \
+		} else {                                     \
+			typ dy = *y2 - y1;                         \
+			( dy > 0) ? dy++ : dy--;                   \
+			*x2 = (typ)( x1 + dy * 1000L / tan);       \
+			*y2 = y1 + dy;                             \
+		}                                            \
+	}                                              \
+}                                                \
+
+ADJUST_LINE_END_DEF(int);
+ADJUST_LINE_END_DEF(LONG);
 
 static Bool
 gp_Arc(
@@ -121,7 +100,7 @@ gp_Arc(
 		HGDIOBJ old;
 
 		old = SelectObject( sys ps, CreatePen( PS_SOLID, 1, sys stylus. brush. lb. lbColor));
-		adjust_line_end( nXRadial1, nYRadial1, &nXRadial2, &nYRadial2, true);
+		adjust_line_end_int( nXRadial1, nYRadial1, &nXRadial2, &nYRadial2);
 		MoveToEx( sys ps, nXRadial1, nYRadial1, nil);
 		ret = LineTo( sys ps, nXRadial2, nYRadial2);
 		DeleteObject(SelectObject( sys ps, old) );
@@ -156,7 +135,7 @@ gp_Chord(
 		}
 
 		old = SelectObject( sys ps, CreatePen( PS_SOLID, 1, sys stylus. brush. lb. lbColor));
-		adjust_line_end( nXRadial1, nYRadial1, &nXRadial2, &nYRadial2, true);
+		adjust_line_end_int( nXRadial1, nYRadial1, &nXRadial2, &nYRadial2);
 		MoveToEx( sys ps, nXRadial1, nYRadial1, nil);
 		ret = LineTo( sys ps, nXRadial2, nYRadial2);
 		DeleteObject(SelectObject( sys ps, old) );
@@ -185,7 +164,7 @@ gp_Pie(
 		old = SelectObject( sys ps, CreatePen( PS_SOLID, 1, sys stylus. brush. lb. lbColor));
 		cx  = ( nLeftRect + nRightRect ) / 2;
 		cy  = ( nTopRect  + nBottomRect ) / 2;
-		adjust_line_end( cx, cy, &nXRadial1, &nYRadial1, true);
+		adjust_line_end_int( cx, cy, &nXRadial1, &nYRadial1);
 		if ( filled ) {
 			if ( nXRadial2 > cx ) nXRadial1 = nXRadial2;
 			if ( nYRadial2 > cy ) nYRadial1 = nYRadial2;
@@ -274,9 +253,10 @@ apc_gp_bars( Handle self, int nr, Rect *rr)
 	int i, y = sys lastSize. y;
 	STYLUS_USE_BRUSH( ps);
 	for ( i = 0; i < nr; i++, rr++) {
-		check_swap( rr->left, rr->right);
-		check_swap( rr->bottom, rr->top);
-		if ( !( ok = Rectangle( ps, rr->left, y - rr->top - 1, rr->left + 2, y - rr->bottom + 1))) {
+    Rect xr = *rr;
+		check_swap( xr.left, xr.right);
+		check_swap( xr.bottom, xr.top);
+		if ( !( ok = Rectangle( ps, xr.left, y - xr.top - 1, xr.left + 2, y - xr.bottom + 1))) {
 			apiErr;
 			break;
 		}
@@ -405,20 +385,31 @@ Bool
 apc_gp_draw_poly( Handle self, int numPts, Point * points)
 {objCheck false;{
 	int i, dy = sys lastSize. y;
-	for ( i = 0; i < numPts; i++)  points[ i]. y = dy - points[ i]. y - 1;
-	if ( points[ 0]. x != points[ numPts - 1].x || points[ 0]. y != points[ numPts - 1].y)
-		adjust_line_end( points[ numPts - 2].x, points[ numPts - 2].y, &points[ numPts - 1].x, &points[ numPts - 1].y, true);
+  POINT *p;
+  Bool ok;
+
+	if ((p = malloc( sizeof(POINT) * numPts)) == nil)
+		return false;
+
+	for ( i = 0; i < numPts; i++)  {
+    p[i]. x = points[i]. x;
+    p[i]. y = dy - points[i]. y - 1;
+  }
+	if ( p[0]. x != p[numPts - 1].x || p[0]. y != p[numPts - 1].y)
+		adjust_line_end_LONG( p[numPts - 2].x, p[numPts - 2].y, &p[numPts - 1].x, &p[numPts - 1].y);
 
 	if (EMULATE_OPAQUE_LINE) {
 		STYLUS_USE_OPAQUE_LINE;
-		Polyline( sys ps, ( POINT*) points, numPts);
+		Polyline( sys ps, p, numPts);
 		STYLUS_RESTORE_OPAQUE_LINE;
 	}
 
 	STYLUS_USE_PEN( sys ps);
-	if ( !Polyline( sys ps, ( POINT*) points, numPts)) apiErrRet;
+	if ( !(ok = Polyline( sys ps, p, numPts))) apiErr;
 
-	return true;
+  free(p);
+
+	return ok;
 }}
 
 Bool
@@ -426,25 +417,35 @@ apc_gp_draw_poly2( Handle self, int numPts, Point * points)
 {objCheck false;{
 	Bool ok = true;
 	int i, dy = sys lastSize. y;
-	DWORD * pts = ( DWORD *) malloc( sizeof( DWORD) * numPts);
+	DWORD * pts;
+  POINT * p;
+
+  pts = ( DWORD *) malloc( sizeof( DWORD) * numPts);
 	if ( !pts) return false;
+	p = malloc( sizeof(POINT) * numPts);
+  if ( !p ) {
+    free(pts);
+    return false;
+  }
 
 	for ( i = 0; i < numPts; i++)  {
-		points[ i]. y = dy - points[ i]. y - 1;
+		p[i]. x = points[i]. x;
+		p[i]. y = dy - points[i]. y - 1;
 		pts[ i] = 2;
 		if ( i & 1)
-			adjust_line_end( points[ i - 1].x, points[ i - 1].y, &points[ i].x, &points[ i].y, true);
+			adjust_line_end_LONG( p[i - 1].x, p[i - 1].y, &p[i].x, &p[i].y);
 	}
 
 	if (EMULATE_OPAQUE_LINE) {
 		STYLUS_USE_OPAQUE_LINE;
-		PolyPolyline( sys ps, ( POINT*) points, pts, numPts/2);
+		PolyPolyline( sys ps, p, pts, numPts/2);
 		STYLUS_RESTORE_OPAQUE_LINE;
 	}
 
 	STYLUS_USE_PEN( sys ps);
-	if ( !( ok = PolyPolyline( sys ps, ( POINT*) points, pts, numPts/2))) apiErr;
+	if ( !( ok = PolyPolyline( sys ps, p, pts, numPts/2))) apiErr;
 	free( pts);
+	free( p);
 	return ok;
 }}
 
@@ -550,18 +551,25 @@ apc_gp_fill_poly( Handle self, int numPts, Point * points)
 {Bool ok = true; objCheck false;{
 	HDC     ps = sys ps;
 	int i,  dy = sys lastSize. y;
+  POINT *p;
 
-	for ( i = 0; i < numPts; i++) points[ i]. y = dy - points[ i]. y - 1;
+  if ((p = malloc( sizeof(POINT) * numPts)) == nil)
+		return false;
+
+	for ( i = 0; i < numPts; i++)  {
+    p[i]. x = points[i]. x;
+    p[i]. y = dy - points[i]. y - 1;
+  }
 
 	if (( sys psFillMode & fmOverlay) == 0) {
 		HGDIOBJ old = SelectObject( ps, hPenHollow);
 		STYLUS_USE_BRUSH( ps);
-		if ( !( ok = Polygon( ps, ( POINT *) points, numPts))) apiErr;
+		if ( !( ok = Polygon( ps, p, numPts))) apiErr;
 		SelectObject( ps, old);
 	} else if ( !stylus_complex( &sys stylus, ps)) {
 		HPEN old = SelectObject( ps, CreatePen( PS_SOLID, 1, sys stylus. brush. lb. lbColor));
 		STYLUS_USE_BRUSH( ps);
-		if ( !( ok = Polygon( ps, ( POINT *) points, numPts))) apiErr;
+		if ( !( ok = Polygon( ps, p, numPts))) apiErr;
 		DeleteObject( SelectObject( ps, old));
 	} else {
 		int dx    = sys lastSize. x;
@@ -575,17 +583,20 @@ apc_gp_fill_poly( Handle self, int numPts, Point * points)
 		trans. x = dx;
 		trans. y = dy;
 		for ( i = 0; i < numPts; i++)  {
-			if ( points[ i]. x > bound. x) bound. x = points[ i]. x;
-			if ( points[ i]. y > bound. y) bound. y = points[ i]. y;
-			if ( points[ i] .x < trans. x) trans. x = points[ i]. x;
-			if ( points[ i] .y < trans. y) trans. y = points[ i]. y;
+			if ( p[ i]. x > bound. x) bound. x = p[ i]. x;
+			if ( p[ i]. y > bound. y) bound. y = p[ i]. y;
+			if ( p[ i] .x < trans. x) trans. x = p[ i]. x;
+			if ( p[ i] .y < trans. y) trans. y = p[ i]. y;
 		}
-		if (( trans. x == dx) || ( trans. y == dy)) return false;
+		if (( trans. x == dx) || ( trans. y == dy)) {
+      free(p);
+      return false;
+    }
 		if ( bound. x > dx) bound. x = dx;
 		if ( bound. y > dy) bound. y = dy;
 		for ( i = 0; i < numPts; i++)  {
-			points[ i]. x -= trans. x;
-			points[ i]. y -= trans. y;
+			p[ i]. x -= trans. x;
+			p[ i]. y -= trans. y;
 		}
 		bound. x -= trans. x - 1;
 		bound. y -= trans. y - 1;
@@ -594,6 +605,7 @@ apc_gp_fill_poly( Handle self, int numPts, Point * points)
 		if ( db) {
 			if ( !( ps = dc_alloc())) { // fact that if dest ps is memory dc, CCB will result mono-bitmap
 				dc_compat_free();
+        free(p);
 				return false;
 			}
 		}
@@ -601,6 +613,7 @@ apc_gp_fill_poly( Handle self, int numPts, Point * points)
 			apiErr;
 			if ( db) dc_free();
 			dc_compat_free();
+      free(p);
 			return false;
 		}
 		if ( db) {
@@ -610,6 +623,7 @@ apc_gp_fill_poly( Handle self, int numPts, Point * points)
 		if ( !( bmMask = CreateBitmap( bound. x, bound. y, 1, 1, nil))) {
 			apiErr;
 			dc_compat_free();
+      free(p);
 			return false;
 		}
 		bmJ = SelectObject( dc, bmSrc);
@@ -622,7 +636,7 @@ apc_gp_fill_poly( Handle self, int numPts, Point * points)
 		SetROP2( dc, R2_WHITE);
 		Rectangle( dc, 0, 0, bound. x, bound. y);
 		SetROP2( dc, R2_BLACK);
-		if ( !( ok = Polygon( dc, ( POINT *) points, numPts))) apiErr;
+		if ( !( ok = Polygon( dc, p, numPts))) apiErr;
 		SelectObject( dc, bmSrc);
 		if ( !( ok &= MaskBlt( ps, trans. x, trans. y, bound. x, bound. y, dc, 0, 0, bmMask, 0, 0,
 					MAKEROP4( 0x00AA0029, rop)))) apiErr;
@@ -631,6 +645,7 @@ apc_gp_fill_poly( Handle self, int numPts, Point * points)
 		DeleteObject( bmMask);
 		DeleteObject( bmSrc);
 	}
+  free(p);
 }return ok;}
 
 
@@ -712,7 +727,7 @@ apc_gp_line( Handle self, int x1, int y1, int x2, int y2)
 {objCheck false;{
 	HDC ps = sys ps;
 
-	adjust_line_end( x1, y1, &x2, &y2, true);
+	adjust_line_end_int( x1, y1, &x2, &y2);
 	y1 = sys lastSize. y - y1 - 1;
 	y2 = sys lastSize. y - y2 - 1;
 
